"""
/***************************************************************************
 MultiEdit
                                 A QGIS plugin
 MultiEdit
                              -------------------
        begin                : 2012-02-17
        copyright            : (C) 2012 by Giuseppe De Marco
        email                : info@pienocampo.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
# Import the PyQt and QGIS libraries
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4 import QtXml
from qgis.core import *
from qgis.gui import *
import pdb

# Initialize Qt resources from file resources.py
import resources
# Import the code for the dialog
from multieditdialog import MultiEditDialog

class MultiEdit:

 def __init__(self, iface):
    # Save reference to the QGIS interface
    self.iface = iface
    # create our GUI dialog
    self.dlg = MultiEditDialog()
    #initialization for select features
    self.turnedoffLayers = []
    self.selectList = []
    self.cLayer = None
    self.provider = None
    self.saved = False

 def initGui(self):
    # Create action that will start plugin configuration
    self.action = QAction(QIcon(":/plugins/MultiEdit/multiedit.png"),"MultiEdit", self.iface.mainWindow())
    # connect the action to the run method
    QObject.connect(self.action, SIGNAL("triggered()"), self.run)
    # Add toolbar button and menu item
    self.iface.addToolBarIcon(self.action)
    self.iface.addPluginToMenu("&Pienocampo ", self.action)

#Custom functions begin-------------------------------------------------------


 #checks if layer are vector type
 def checkvector(self):
    count = 0
    for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
        if layer.type() == QgsMapLayer.VectorLayer:
            count += 1
            return count

 #choose layer to process	
 def chooselayer(self):
    self.dlg.ui.chosenlayer.clear()
    self.dlg.ui.txtFeedBack.clear()
    self.dlg.ui.newvalue.clear()
    for name, layer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
        if layer.type() == QgsMapLayer.VectorLayer:
            self.dlg.ui.chosenlayer.addItem(layer.name())

 #choose attribute of present layers 	
 def set_select_attributes(self):
    self.dlg.ui.Column.clear()
    self.dlg.ui.new_field.clear()
    if self.dlg.ui.chosenlayer.currentText() != "":
        layername = self.dlg.ui.chosenlayer.currentText()
        for name, selectlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
            if selectlayer.name() == layername:
                for index, field in selectlayer.dataProvider().fields().iteritems():
                    self.dlg.ui.Column.addItem(field.name())
                    self.dlg.ui.new_field.addItem(field.name())
    
 #choose attribute value
 def set_select_value(self):
    self.dlg.ui.oldattribute.clear()
    column = self.dlg.ui.Column.currentText()
    layername = self.dlg.ui.chosenlayer.currentText()
    output = ""
    #select layer
    for name, selectlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
        if selectlayer.name() == layername:	
            provider = selectlayer.dataProvider()	
            #select field and fetch values
            for index, field in provider.fields().iteritems():
                if field.name() == column:
                    feat = QgsFeature()
                    while provider.nextFeature(feat):
                        attrs = feat.attributeMap()
                        for (k,attr) in attrs.iteritems():
                            self.dlg.ui.oldattribute.addItem(attr.toString())
                            #output += "\n" + attr.toString()				
                            #self.dlg.ui.txtFeedBack.setText(output)	
 
 #find unique values in layer feature classification
 def set_unique_value(self):
    #pyqtRemoveInputHook()
    #pdb.set_trace()
    provider = None
    self.dlg.ui.oldattribute.clear()
    uniquelayer = self.dlg.ui.chosenlayer.currentText()
    uniquecolumn = self.dlg.ui.Column.currentText()
    for name, selectlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
        if selectlayer.name() == uniquelayer:	
            uniqueprovider = selectlayer.dataProvider()
            if (uniqueprovider):
                for index, field in uniqueprovider.fields().iteritems():
                    if field.name() == uniquecolumn:
                        values = uniqueprovider.uniqueValues(index)
                        for value in values:
                            self.dlg.ui.oldattribute.addItem(value.toString())

 #Select features to process
 def select_features(self):
    #pyqtRemoveInputHook()
    #pdb.set_trace()
    #clear any preselected feature...
    self.selectList = []
    #let's begin
    #self.selectList = []
    currlayer = self.dlg.ui.chosenlayer.currentText()
    currcolumn = self.dlg.ui.Column.currentText()
    currfeature = self.dlg.ui.oldattribute.currentText()
    #turn selected layer off
    self.turn_layer_off(currlayer)
    nsel = 0
    #select data provider and layer
    for name, selectlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
        if selectlayer.name() == currlayer:	
            self.cLayer = selectlayer
            self.provider = selectlayer.dataProvider()
            if self.cLayer:
                for index, field in self.provider.fields().iteritems():
                    if field.name() == currcolumn:
                        feat = QgsFeature()
                        #navigates through the features and select those whose name 
                        #correspond to the value in combo box
                        while self.provider.nextFeature(feat):
                            attrs = feat.attributeMap()
                            for (k,attr) in attrs.iteritems():
                                if attr.toString() == currfeature:
                                    # append id to select to list
                                    self.selectList.append(feat.id())
         
    # make the actual selection
    if self.selectList:
        self.cLayer.setSelectedFeatures(self.selectList)
        nsel = self.cLayer.selectedFeatureCount()
        #some info in the text browser to know what's going on
        self.dlg.ui.txtFeedBack.setText(str(nsel)+" Feature/s selected"+"\nin Layer--> " + self.cLayer.name() + "\nin Field--> " + currcolumn + "\nValue to be modified--> " + currfeature)
 
#first change value function begin--------------------------------------------------------------------------------
 #change feature value to the same field: first function now deprecated	
 #def changevalue(self):
  #  #pyqtRemoveInputHook()
   # #pdb.set_trace()
    #layername = self.dlg.ui.chosenlayer.currentText()
    #val = self.get_new_value()
    #col = self.dlg.ui.Column.currentText()
    #if val == "":
    #    QMessageBox.warning(None, "Warning","New attribute value is empty or null \n  Please type in one!")
    #    return
    #else: 	
    #    #initialize variables and retrieve column number in Attribute table 
        #to change values accordingly in the right column 
    #    nF = 0
    #    ncol = 0
    #    for name, selectlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
    #        if selectlayer.name() == layername:
    #            layer = selectlayer
    #            fields = layer.dataProvider().fields()
    #            type (fields)
    #            for key,value in fields.items():
    #                if value.name() == col:
    #                    ncol = key
    #            if(layer):
    #                nF = layer.selectedFeatureCount()
    #                #some information in the textBrowser
    #                self.dlg.ui.txtFeedBack.setText("Filed name and ID--> " + col +" - "+ str(ncol) + "\nNumber of Features to modify--> "+ str(nF))
                    #the real work now
    #                if (nF > 0):
    #                    layer.startEditing()
    #                    ob = layer.selectedFeaturesIds()
    #                    b = QVariant(val)
    #                   if (nF > 1):
    #                        for i in ob:
    #                            layer.changeAttributeValue(int(i),ncol,b) # ncol being the nth column
    #                    else:
    #                        layer.changeAttributeValue(int(ob[0]),ncol,b) # ncol being the nth column
    #                        #layer.commitChanges() ''buggy for some reasons and besides not committing changes
                            # leaves a rollback chance''
    #                else:
    #                    QMessageBox.critical(None,"Error", "Please select at least one feature from current layer")
     #           else:
     #               QMessageBox.critical(None,"Error","Please select a layer")
#first change value function end--------------------------------------------------------------------------------

 def change_to_any(self):#change values to any column of the attribute table

    layername = self.dlg.ui.chosenlayer.currentText()
    val = self.get_new_value()
    col_new = self.dlg.ui.new_field.currentText()
    if val == "":
        QMessageBox.warning(None, "Warning","New attribute value is empty or null \n  Please type in one!")
        return
    else: 	
        #initialize variables and retrieve column number in Attribute table 
        #to change values in a chosen field 
        nF = 0
        ncol_new = 0
        for name, selectlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
            if selectlayer.name() == layername:
                layer = selectlayer
                fields = layer.dataProvider().fields()
                type (fields)
                for key,value in fields.items():
                    if value.name() == col_new:
                        ncol_new = key
                if(layer):
                    nF = layer.selectedFeatureCount()
                    #some information in the textBrowser
                    self.dlg.ui.txtFeedBack.setText("Field name and ID--> " + col_new + " - " + str(ncol_new) + "\nNumber of Features to modify--> "+ str(nF))
                    #the real work now
                    if (nF > 0):
                        layer.startEditing()
                        ob = layer.selectedFeaturesIds()
                        b = QVariant(val)
                        if (nF > 1):
                            for i in ob:
                                layer.changeAttributeValue(int(i),ncol_new,b) # ncol_new being the nth column
                        else:
                            layer.changeAttributeValue(int(ob[0]),ncol_new,b) # ncol_new being the nth column
                            #layer.commitChanges() #buggy for some reasons and besides not committing changes
                            # leaves a rollback chance'
                    else:
                        QMessageBox.critical(None,"Error", "Please select at least one feature from current layer")
                else:
                    QMessageBox.critical(None,"Error","Please select a layer")

 #Clear selected vector features 
 def clearselection(self):
    self.dlg.ui.txtFeedBack.clear()
    clearlist = []
    for name, selectlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
        if selectlayer.type() == QgsMapLayer.VectorLayer:
            selectlayer.setSelectedFeatures(clearlist)

 def show_table(self):
    #shows attribute table of chosen layer 
    table_layer=self.dlg.ui.chosenlayer.currentText()
    for name, selectlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
        if selectlayer.name() == table_layer:
            show_layer_t = selectlayer
            self.iface.showAttributeTable(show_layer_t)
            
    
 def get_new_value(self):
    new_val = self.dlg.ui.newvalue.toPlainText()
    return new_val

 def turn_layer_off(self, loadedlayer):
    for name, selectlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
        if selectlayer.name() == loadedlayer:
            theloadedlayer = selectlayer
            legend = self.iface.legendInterface()
            #access legendInteface class to determine if a layer is visible
            #then set it unvisible and add layer to a list
            if (legend.isLayerVisible(theloadedlayer)):
                legend.setLayerVisible(theloadedlayer, False)
                self.turnedoffLayers.append(selectlayer)
        
 def turn_layer_on (self, unloadedlayer):
    #access legendInterface class and turn unloadelayer layer on
    legend = self.iface.legendInterface()
    legend.setLayerVisible(unloadedlayer, True)

 def restore_visibility(self):
    #pick turned off layers froma a list and set them visible
    if self.turnedoffLayers != []:
        for i in range(len(self.turnedoffLayers)):
            self.turn_layer_on(self.turnedoffLayers[i])

 def exit(self):
    #put rest of the code here
    self.clearselection()
            
    #turn on layers turned off by turn_layer_off f(x) stored in a list
    #globally defined called turnedoffLayers, this method preserves the list, if one needs to purge items
    #one should use this, given a the list: while a: function(a.pop)
    #while self.turnedoffLayers:
    #   self.turn_layer_on(self.turnedoffLayers.pop)
    self.restore_visibility()
    #self.dlg.ui.newvalue.clear()
    if (self.saved == False):
        QMessageBox.information(None,"INFO","Remember to review the changes \n and then save the edits. Thank you")
    #self.iface.zoomToPrevious()
    return

 def save_edits(self):
    self.saved = False
    #chose layer name to check in iteration
    layername = self.dlg.ui.chosenlayer.currentText()
    #browse layers in project to find matching items, select layer and if started editing
    #commit changes...
    for name, selectlayer in QgsMapLayerRegistry.instance().mapLayers().iteritems():
        if selectlayer.name() == layername:
            if (selectlayer.isEditable()):
                selectlayer.commitChanges()
                #show result in textBrowser
                self.dlg.ui.txtFeedBack.setText("Edits saved in Layer--> "+ layername)
                self.saved = True
    return self.saved
                

#Custom function end-------------------------------------------------------------------------------------------------------------------
	
		
 def unload(self):
    # Remove the plugin menu item and icon
    self.iface.removePluginMenu("&MultiEdit ",self.action)
    self.iface.removeToolBarIcon(self.action)
    
 # run method that performs all the real work----------------------------------------------------------------------

 def run(self):
    #pyqtRemoveInputHook()
    #pdb.set_trace()
    #initial check if no vector layer no party...
    #check = None
    check = self.checkvector()
    if check == None:
        QMessageBox.critical(None, "Critical","No vector layers \n Please load some, then reload plugin")
        return
    else:
        boolvar ="YES"
        #Initial comboboxes filling
        self.chooselayer()
        #pyqtRemoveInputHook()
        #pdb.set_trace()
        #Connect to change in layer combobox and column combobox and execute
        #information retrieving procedures
        QObject.connect(self.dlg.ui.chosenlayer, SIGNAL("currentIndexChanged(QString)"), self.set_select_attributes)
        self.set_select_attributes()
        #every change in layer choice clears selection anyway
        QObject.connect(self.dlg.ui.chosenlayer, SIGNAL("currentIndexChanged(QString)"), self.clearselection)
        #populate attribute values combobox
        #2nd version experimental with unique values
        self.set_unique_value()
        #Connection for 2nd version
        QObject.connect(self.dlg.ui.Column, SIGNAL("currentIndexChanged(QString)"), self.set_unique_value)
        #Connection for 1st version
        #QObject.connect(self.dlg.ui.Column, SIGNAL("currentIndexChanged(QString)"), self.set_select_value)
        #self.set_select_value()
        #Plaintext event
        QObject.connect(self.dlg.ui.newvalue, SIGNAL("currentIndexChanged(QString)"), self.get_new_value)		
        #Buttons events
        QObject.connect(self.dlg.ui.select, SIGNAL("clicked(bool)"), self.select_features)
        QObject.connect(self.dlg.ui.unselect, SIGNAL("clicked(bool)"), self.clearselection)
        QObject.connect(self.dlg.ui.save, SIGNAL("clicked(bool)"), self.save_edits)
        QObject.connect(self.dlg.ui.show_t, SIGNAL("clicked(bool)"), self.show_table)
        #older change function 
        #QObject.connect(self.dlg.ui.change, SIGNAL("clicked(bool)"), self.changevalue)#the real thing...
        QObject.connect(self.dlg.ui.change_another, SIGNAL("clicked(bool)"), self.change_to_any)#the real thing 2...
        #1st version without unique values
        #self.set_select_value()
        #pyqtRemoveInputHook()
        #pdb.set_trace()
        self.dlg.show()
        self.dlg.ui.txtFeedBack.setText("VECTOR LAYER? -->"+ boolvar)
        QObject.connect(self.dlg.ui.Exit, SIGNAL("clicked(bool)"), self.exit)
        
        #full extent workaround for select function, future maybe to turn layers off before selecting...done!
        #self.iface.zoomFull()
        
